<?php

namespace Drupal\datafield\Plugin\DataField\FieldFormatter;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\datafield\Plugin\DataFieldFormatterInterface;
use Drupal\file\Entity\File;

/**
 * Formatter to render the file URI to its download path.
 *
 * @FieldFormatter(
 *   id = "file_uri",
 *   label = @Translation("File URI"),
 *   field_types = {
 *     "uri",
 *     "file",
 *   }
 * )
 */
class FileUriFormatter implements DataFieldFormatterInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $settings['file_download_path'] = FALSE;
    return $settings;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $settings = $form['#settings'];
    $element['file_download_path'] = [
      '#title' => $this->t('Display the file download URI'),
      '#type' => 'checkbox',
      '#default_value' => $settings['file_download_path'] ?? FALSE,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary($settings = []) {
    $summary = [];
    $settings += self::defaultSettings();
    if ($settings['file_download_path']) {
      $summary[] = $this->t('Display the file download URI');
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements($item, $langcode) {
    if (empty($item->value) || !is_numeric($item->value)) {
      return $item->value;
    }
    $fid = $item->value;
    $fileUrlGenerator = \Drupal::service('file_url_generator');
    $file = File::load($fid);
    if (empty($file)) {
      return NULL;
    }
    $url = $fileUrlGenerator->generate($file->getFileUri());
    $fieldName = $file->getFilename();
    $ext = pathinfo($fieldName, PATHINFO_EXTENSION);

    $title = new FormattableMarkup('<i class="bi bi-filetype-@ext"></i> ' . $fieldName, [
      '@ext' => strtolower($ext),
    ]);
    return [
      '#type' => 'link',
      '#langcode' => $langcode,
      '#title' => $title,
      '#url' => $url,
      '#cache' => ['tags' => $file->getCacheTags()],
    ];
  }

}
