<?php

namespace Drupal\datafield\Plugin\DataField\FieldFormatter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\datafield\Plugin\DataFieldFormatterInterface;

/**
 * Plugin implementation of the 'image_url' formatter.
 *
 * @FieldFormatter(
 *   id = "image_url",
 *   label = @Translation("URL Image"),
 *   field_types = {
 *     "image",
 *     "file",
 *   }
 * )
 */
class ImageUrlFormatter implements DataFieldFormatterInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'image_style' => 'medium',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $image_styles = image_style_options(FALSE);
    $description_link = Link::fromTextAndUrl(
      $this->t('Configure Image Styles'),
      Url::fromRoute('entity.image_style.collection')
    );
    $subfield = $form['#machine_name'];
    $element['image_style'] = [
      '#title' => $this->t('Image style'),
      '#type' => 'select',
      '#default_value' => $settings['image_style'] ?? self::defaultSettings()['image_style'],
      '#empty_option' => $this->t('None (original image)'),
      '#options' => $image_styles,
      '#description' => $description_link->toRenderable() + [
        '#access' => \Drupal::currentUser()->hasPermission('administer image styles'),
      ],
      '#states' => [
        'visible' => [
          'select[name$="[' . $subfield . '][plugin_type]"]' => [
            ['value' => 'image'],
            ['value' => 'image_url'],
          ],
        ],
      ],
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary($settings = []) {
    $summary = [];

    $image_styles = image_style_options(FALSE);
    // Unset possible 'No defined styles' option.
    unset($image_styles['']);
    // Styles could be lost because of enabled/disabled modules that defines
    // their styles in code.
    $image_style_setting = $settings['image_style'] ?? self::defaultSettings()['image_style'];
    if (isset($image_styles[$image_style_setting])) {
      $summary[] = $this->t('Image style: @style', ['@style' => $image_styles[$image_style_setting]]);
    }
    else {
      $summary[] = $this->t('Original image');
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements($item, $langcode) {
    if (empty($item->value) || !is_numeric($item->value)) {
      return $item->value;
    }
    $fid = $item->value;
    $fileUrlGenerator = \Drupal::service('file_url_generator');
    $entity_type_manager = \Drupal::service('entity_type.manager');
    $file = $entity_type_manager->getStorage('file')->load($fid);
    if (empty($file)) {
      return $item->value;
    }
    $image_uri = $file->getFileUri();
    $image_style = $entity_type_manager->getStorage('image_style')->load($item->settings['image_style']);
    $url = $image_style ? $fileUrlGenerator->transformRelative($image_style->buildUrl($image_uri)) : $fileUrlGenerator->generateString($image_uri);
    return [
      '#markup' => $url,
      '#cache' => ['tags' => $file->getCacheTags()],
    ];
  }

}
