<?php

namespace Drupal\datafield\Plugin\DataField\FieldFormatter;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\datafield\Plugin\DataFieldFormatterInterface;

/**
 * Plugin implementation of the 'timestamp' formatter.
 *
 * @FieldFormatter(
 *   id = "timestamp",
 *   label = @Translation("Default"),
 *   field_types = {
 *     "datetime_iso8601",
 *     "date",
 *   }
 * )
 */
class TimestampFormatter implements DataFieldFormatterInterface {
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'date_format' => 'medium',
      'custom_date_format' => '',
      'timezone' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $settings = $form['#settings'];
    $subfield = $form['#machine_name'];

    $format_types = \Drupal::entityTypeManager()->getStorage('date_format')->loadMultiple();
    $dateFormatter = \Drupal::service('date.formatter');
    $time = new DrupalDateTime();
    $options['custom'] = $this->t('Custom date format');
    foreach ($format_types as $type => $type_info) {
      $format = $dateFormatter->format($time->getTimestamp(), $type);
      $options[$type] = $type_info->label() . ' (' . $format . ')';
    }
    $elements['date_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Date format'),
      '#description' => $this->t('Choose a format for displaying the date.'),
      '#options' => $options,
      '#default_value' => $settings['date_format'] ?? '',
    ];
    $visible = 'select[name*="[settings_edit_form][settings][formatter_settings][' . $subfield . '][date_format]"]';
    $elements['custom_date_format'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom date format'),
      '#description' => $this->t('See <a href="https://www.php.net/manual/datetime.format.php#refsect1-datetime.format-parameters" target="_blank">the documentation for PHP date formats</a>.'),
      '#default_value' => $settings['custom_date_format'] ?? '',
      '#states' => [
        'visible' => [$visible => ['value' => 'custom']],
      ],
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary($settings = []) {
    $summary = [];
    $date_format = $settings['date_format'] ?? NULL;
    $summary[] = $this->t('Date format: @date_format', ['@date_format' => $date_format]);
    if (!empty($settings['date_format']) && $settings['date_format'] === 'custom' && ($custom_date_format = $settings['custom_date_format'])) {
      $summary[] = $this->t('Custom date format: @custom_date_format', ['@custom_date_format' => $custom_date_format]);
    }
    if ($timezone = $settings['timezone'] ?? NULL) {
      $summary[] = $this->t('Time zone: @timezone', ['@timezone' => $timezone]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements($item, $langcode) {
    if (empty($item->value)) {
      return $item->value;
    }
    $isDate = in_array($item->storage["type"], ['date', 'datetime_iso8601']);
    if (empty($item->settings) && $isDate) {
      $valueTypeText = ['time', 'year', 'month', 'week'];
      if (in_array($item->storage["datetime_type"], $valueTypeText)) {
        return [
          '#cache' => [
            'contexts' => [
              'timezone',
            ],
          ],
          '#markup' => $item->value,
        ];
      }
    }
    $date_format = $item->settings['date_format'] ?? self::defaultSettings()['date_format'];
    $custom_date_format = '';
    $timezone = $item->settings['timezone'] ?? NULL;
    $dateFormatter = \Drupal::service('date.formatter');
    // If an RFC2822 date format is requested, then the month and day have to
    // be in English. @see http://www.faqs.org/rfcs/rfc2822.html
    if ($date_format === 'custom' && ($custom_date_format = $item->settings['custom_date_format']) === 'r') {
      $langcode = 'en';
    }
    $timestamp = is_numeric($item->value) ? $item->value : strtotime($item->value);
    return [
      '#cache' => [
        'contexts' => [
          'timezone',
        ],
      ],
      '#markup' => $dateFormatter->format($timestamp, $date_format, $custom_date_format, $timezone, $langcode),
    ];
  }

}
