<?php

namespace Drupal\datafield\Plugin\DataField\FieldWidget;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\datafield\Plugin\DataFieldWidgetInterface;
use Drupal\link\LinkItemInterface;
use Symfony\Component\Validator\ConstraintViolationInterface;

/**
 * Plugin implementation of the 'path' widget.
 *
 * @FieldWidget(
 *   id = "path",
 *   label = @Translation("URL alias"),
 *   field_types = {
 *     "uri",
 *   }
 * )
 */
class PathWidget implements DataFieldWidgetInterface {
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  private static function defaultSettings() {
    return [
      'target_type' => 'node',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormElement(&$element, $item = NULL, $setting = []) {
    $element['#type'] = 'entity_autocomplete';
    $element['#target_type'] = $element["#widget_settings"]["target_type"] ?? self::defaultSettings()['target_type'];
    $element['#attributes']['data-autocomplete-first-character-blacklist'] = '/#?';
    $element['#link_type'] = LinkItemInterface::LINK_GENERIC;
    $element['#process_default_value'] = FALSE;
    if (is_numeric($element['#default_value'])) {
      $entity = \Drupal::entityTypeManager()
        ->getStorage($element['#target_type'])
        ->load($element['#default_value']);
      $label = $entity->label();
      $element['#default_value'] = $label . ' (' . $element['#default_value'] . ')';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function errorElement(array $element, ConstraintViolationInterface $violation, array $form, FormStateInterface $form_state) {
    $path = explode('.', $violation->getPropertyPath());
    $subfield = end($path);
    if (!empty($element[$subfield])) {
      $settings = $element[$subfield]['#field_settings'];
      if (!empty($settings['list'])) {
        return FALSE;
      }
    }
    if (!empty($element[$subfield]['#value'])) {
      $pattern = '/\((\d+)\)/';
      $matches = [];
      preg_match_all($pattern, $element[$subfield]['#value'], $matches);
      if (!empty($matches[1])) {
        $last = end($matches[1]);
        if (is_numeric($last)) {
          return FALSE;
        }
      }
    }
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $widget_settings = $form['#settings'];
    $repository = \Drupal::service('entity_type.repository')
      ->getEntityTypeLabels(TRUE);
    $contentType = $repository['Content'] ?? [];
    return [
      'target_type' => [
        '#type' => 'select',
        '#title' => $this->t('Entity reference type'),
        '#options' => $contentType,
        '#empty_option' => $this->t('- Select a field type -'),
        '#default_value' => $widget_settings['target_type'] ?? self::defaultSettings()['target_type'],
      ],
    ];
  }

}
