<?php

namespace Drupal\datafield\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'data_field' field type.
 *
 * @FieldType(
 *   id = "data_field",
 *   label = @Translation("Data Field"),
 *   description = @Translation("Data Field."),
 *   default_widget = "data_field_table_widget",
 *   default_formatter = "data_field_table_formatter"
 * )
 */
class DataField extends FieldItemBase {

  public const DATETIME_STORAGE_TIMEZONE = 'UTC';

  public const DATETIME_DATETIME_STORAGE_FORMAT = 'Y-m-d\TH:i:s';

  public const DATETIME_DATE_STORAGE_FORMAT = 'Y-m-d';

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    // Need to have at least one item by default because the table is created
    // before the user gets a chance to customize and will throw an Exception
    // if there isn't at least one column defined.
    return [
      'columns' => [
        'value' => [
          'name' => 'value',
          'max_length' => 255,
          'size' => 'normal',
          'type' => 'string',
          'unsigned' => FALSE,
          'precision' => 10,
          'scale' => 2,
          'datetime_type' => 'date',
        ],
      ],
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data): array {
    $settings = $this->getSettings();
    $default_settings = self::defaultStorageSettings()['columns']['value'];
    if ($form_state->isRebuilding()) {
      $settings = $form_state->getValue('settings');
    }
    else {
      $settings['storage'] = $settings['columns'];
    }

    // Add a new item if there aren't any, or we're rebuilding.
    if ($form_state->get('add') || count($settings['columns']) == 0) {
      $settings['columns'][] = current($settings['columns']);
      $form_state->set('add', NULL);
    }

    $wrapper_id = 'datafield-items-wrapper';
    $element = [
      '#tree' => TRUE,
      'columns' => [
        '#type' => 'value',
        '#value' => $settings['columns'],
      ],
      'storage' => [
        '#type' => 'table',
        '#prefix' => '<div id="' . $wrapper_id . '">',
        '#suffix' => '</div>',
        '#header' => [
          $this->t('Machine-readable name'),
          $this->t('Type'),
          $this->t('Maximum length'),
          $this->t('Unsigned'),
          $this->t('Size'),
          $this->t('Precision'),
          $this->t('Scale'),
          $this->t('Date type'),
          '',
        ],
      ],
      'actions' => [
        '#type' => 'actions',
      ],
    ];
    $fieldName = $this->getFieldDefinition()->getName();
    $lengthField = strlen($fieldName);
    $i = 0;
    foreach ($settings['columns'] as $subfield => $item) {
      if ($i === $form_state->get('remove')) {
        $form_state->set('remove', NULL);
        // @todo it must remove columns in database.
        if (!empty($settings["columns"][$subfield])) {
          unset($settings["columns"][$subfield]);
        }
        if (!empty($element["columns"]["#value"][$subfield])) {
          unset($element["columns"]["#value"][$subfield]);
        }
        if (!empty($element["columns"]["#value"][$subfield])) {
          unset($element["columns"]["#value"][$subfield]);
        }
        if (!empty($element["storage"][$i])) {
          unset($element["storage"][$i]);
        }
        continue;
      }

      $element['storage'][$i]['name'] = [
        '#type' => 'machine_name',
        '#description' => $this->t('A unique machine-readable name containing only letters, numbers, or underscores. This will be used in the column name on the field table in the database.'),
        '#default_value' => $item['name'] ?? uniqid('value_'),
        '#disabled' => $has_data,
        '#maxlength' => 63 - $lengthField,
        '#required' => TRUE,
        '#title_display' => 'invisible',
        '#attributes' => [
          'pattern' => '[a-z_0-9]+',
        ],
        '#machine_name' => [
          'exists' => [$this, 'machineNameExists'],
          'standalone' => TRUE,
        ],
      ];

      $element['storage'][$i]['type'] = [
        '#type' => 'select',
        '#default_value' => $item['type'],
        '#disabled' => $has_data,
        '#required' => TRUE,
        '#options' => $this->subfieldTypes(),
      ];

      $element['storage'][$i]['max_length'] = [
        '#type' => 'number',
        '#description' => $this->t('The maximum length of the subfield in characters.'),
        '#default_value' => $item['max_length'] ?? $default_settings["max_length"],
        '#disabled' => $has_data,
        '#min' => 1,
        '#states' => [
          'visible' => [
            ":input[name='settings[storage][$i][type]']" => [
              ['value' => 'string'],
              ['value' => 'char'],
              ['value' => 'varchar'],
            ],
          ],
        ],
      ];

      $element['storage'][$i]['unsigned'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Unsigned'),
        '#default_value' => $item['unsigned'] ?? $default_settings['unsigned'],
        '#disabled' => $has_data,
        '#states' => [
          'visible' => [
            ':input[name="settings[storage][' . $i . '][type]"]' => [
              ['value' => 'integer'],
              ['value' => 'float'],
              ['value' => 'numeric'],
            ],
          ],
        ],
      ];

      $element['storage'][$i]['size'] = [
        '#type' => 'select',
        '#default_value' => $item['size'] ?? $default_settings['size'],
        '#disabled' => $has_data,
        '#options' => [
          'normal' => $this->t('Normal'),
          'big' => $this->t('Big'),
          'medium' => $this->t('Medium'),
          'small' => $this->t('Small'),
          'tiny' => $this->t('Tiny'),
        ],
        '#states' => [
          'visible' => [
            ':input[name="settings[storage][' . $i . '][type]"]' => [
              ['value' => 'integer'],
              ['value' => 'serial'],
              ['value' => 'float'],
              ['value' => 'text'],
              ['value' => 'blob'],
            ],
          ],
        ],
      ];

      $element['storage'][$i]['precision'] = [
        '#type' => 'number',
        '#description' => $this->t('The total number of digits to store in the database, including those to the right of the decimal.'),
        '#default_value' => $item['precision'] ?? $default_settings['precision'],
        '#disabled' => $has_data,
        '#min' => 10,
        '#max' => 32,
        '#states' => [
          'visible' => [
            ":input[name='settings[storage][$i][type]']" => [
              ['value' => 'numeric'],
              ['value' => 'float'],
            ],
          ],
        ],
      ];

      $element['storage'][$i]['scale'] = [
        '#type' => 'number',
        '#description' => $this->t('The number of digits to the right of the decimal.'),
        '#default_value' => $item['scale'] ?? $default_settings['scale'],
        '#disabled' => $has_data,
        '#min' => 0,
        '#max' => 10,
        '#states' => [
          'visible' => [":input[name='settings[storage][$i][type]']" => ['value' => 'numeric']],
        ],
      ];

      $element['storage'][$i]['datetime_type'] = [
        '#type' => 'radios',
        '#description' => $this->t('Choose the type of date to create.'),
        '#default_value' => $item['datetime_type'] ?? $default_settings['datetime_type'],
        '#disabled' => $has_data,
        '#options' => [
          'datetime' => $this->t('Date and time'),
          'date' => $this->t('Date'),
          'time' => $this->t('Time'),
          'year' => $this->t('Year'),
          'month' => $this->t('Month'),
          'week' => $this->t('Week'),
          'timestamp' => $this->t('Timestamp'),
        ],
        '#states' => [
          'visible' => [
            ":input[name='settings[storage][$i][type]']" => [
              ['value' => 'datetime_iso8601'],
              ['value' => 'date'],
            ],
          ],
        ],
      ];

      $element['storage'][$i]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#submit' => [get_class($this) . '::removeSubmit'],
        '#name' => 'remove:' . $i,
        '#delta' => $i,
        '#disabled' => $has_data,
        '#ajax' => [
          'callback' => [$this, 'actionCallback'],
          'wrapper' => $wrapper_id,
        ],
      ];
      $i++;
    }

    if (!$has_data) {
      $element['actions']['add'] = [
        '#type' => 'submit',
        '#value' => $this->t('Add another'),
        '#submit' => [get_class($this) . '::addSubmit'],
        '#ajax' => [
          'callback' => [$this, 'actionCallback'],
          'wrapper' => $wrapper_id,
        ],
        // @todo copy setting.
        '#states' => [
          'visible' => [
            'select[data-id="datafield-settings-clone"]' => ['value' => ''],
          ],
        ],
      ];
    }

    $form_state->setCached(FALSE);
    return $element;
  }

  /**
   * Submit handler for the StorageConfigEditForm.
   *
   * This handler is added in data field. Module since it has to be placed
   * directly on the submit button (which we don't have access to in our
   * ::storageSettingsForm() method above).
   */
  public static function submitStorageConfigEditForm(array &$form, FormStateInterface $form_state) {
    // Re-key our column settings and overwrite the values in form_state so that
    // we have clean settings saved to the db.
    $columns = [];
    foreach ($form_state->getValue(['settings', 'storage']) as $item) {
      $columns[$item['name']] = $item;
      unset($columns[$item['name']]['remove']);
    }
    $form_state->setValue(['settings', 'columns'], $columns);
    $form_state->setValue(['settings', 'storage'], NULL);

    // Reset the field storage config property - it will be recalculated
    // when accessed via the property definition getter.
    // @see Drupal\field\Entity\FieldStorageConfig::getPropertyDefinitions()
    // If we don't do this, an exception is thrown during the table update that
    // is very difficult to recover from since the original field tables have
    // already been removed at that point.
    $field_storage_config = $form_state->getBuildInfo()['callback_object']->getEntity();
    $field_storage_config->set('propertyDefinitions', NULL);
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'field_settings' => [],
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state): array {
    $settings = $this->getSettings();
    if ($form_state->isRebuilding()) {
      $formSettings = $form_state->getValue('settings');
      if (empty($formSettings)) {
        $formSettings = $form_state->getUserInput()['settings'];
      }
      $field_settings = $formSettings['field_settings'] ?? [];
      $settings['field_settings'] = $field_settings;
    }
    else {
      $field_settings = $this->getSetting('field_settings');
    }
    if (!empty($settings["field_settings"])) {
      unset($settings["field_settings"]);
    }
    $types = static::subfieldTypes();
    $element = [
      '#type' => 'fieldset',
      '#title' => $this->t('Data Items'),
    ];
    $pluginService = \Drupal::service('plugin.manager.data_field.type');
    $fieldName = $this->getFieldDefinition()->getName();
    $entityType = $this->getEntity()->getEntityTypeId();
    foreach ($settings["columns"] as $subfield => $item) {
      $type = $item['type'];
      $plugin = $pluginService->createInstance($type);
      $title = ucfirst(str_replace('_', ' ', $subfield));
      if (!empty($field_settings[$subfield]['label'])) {
        $title = $field_settings[$subfield]['label'];
      }
      $title .= ' - ' . $types[$type];
      $field_settings[$subfield]['machine_name'] = $subfield;
      $field_settings[$subfield]['field_name'] = $fieldName;
      $field_settings[$subfield]['entity_type'] = $entityType;
      $element['field_settings'][$subfield] = [
        '#type' => 'details',
        '#title' => $title,
        '#open' => FALSE,
        '#tree' => TRUE,
      ] + $plugin->fieldSettingsForm($field_settings[$subfield], $item);

    }

    return $element;
  }

  /**
   * {@inheritdoc}
   *
   * @todo Find a way to disable constraints for default field values.
   */
  public function getConstraints(): array {
    $constraint_manager = \Drupal::typedDataManager()
      ->getValidationConstraintManager();
    $constraints = parent::getConstraints();
    $settings = $this->getSettings();
    $subConstraints = [];
    $pluginService = \Drupal::service('plugin.manager.data_field.type');
    foreach ($this->getSetting('columns') as $subfield => $item) {
      $subfield_type = $item['type'];
      $field_settings = $settings['field_settings'][$subfield] ?? [];
      $field_settings += $item;
      $plugin = $pluginService->createInstance($subfield_type);
      $subConstraints[$subfield] = $plugin->getConstraints($field_settings);
    }
    $constraints[] = $constraint_manager->create('ComplexData', $subConstraints);
    return $constraints;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {
    $pluginService = \Drupal::service('plugin.manager.data_field.type');
    $columns = [];
    foreach ($field_definition->getSetting('columns') as $subfield => $item) {
      if (!empty($item['name'])) {
        $subfield = $item['name'];
      }
      $type = $item['type'];
      $plugin = $pluginService->createInstance($type);
      $schema = $plugin->schema($item);
      $schema['columns']['value']['description'] = ucfirst($subfield) . ' subfield value.';
      $columns[$subfield] = $schema['columns']['value'];
    }

    return ['columns' => $columns];
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties = [];
    $pluginService = \Drupal::service('plugin.manager.data_field.type');
    foreach ($field_definition->getSetting('columns') as $item) {
      $type = $item['type'] ?? 'string';
      $plugin = $pluginService->createInstance($type);
      $properties[$item['name']] = $plugin->propertyDefinitions($item);
    }

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    $settings = $this->getSettings();
    if (empty($columns = $settings["columns"])) {
      return FALSE;
    }
    foreach ($columns as $subfield => $item) {
      if ($item['type'] == 'boolean') {
        // Booleans can be 1 or 0.
        if ($this->{$subfield} == 1) {
          return FALSE;
        }
      }
      elseif ($this->{$subfield} !== NULL && $this->{$subfield} !== '') {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * Returns available subfield storage types.
   */
  public static function subfieldTypes(): array {
    $types = \Drupal::service('plugin.manager.data_field.type');
    $plugin_definitions = $types->getDefinitions();
    $result = [];
    array_walk($plugin_definitions, function ($object) use (&$result) {
      $result[$object["id"]] = $object["label"];
    });
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public static function fieldSettingsToConfigData(array $settings): array {
    foreach ($settings['field_settings'] as $subfield => $item) {
      if (empty($item['allowed_values'])) {
        continue;
      }
      $structured_values = [];
      if (is_string($item['allowed_values'])) {
        $explode = explode("\r", str_replace("\n", '', $item['allowed_values']));
        $item['allowed_values'] = $explode;
        if (!empty($explode)) {
          $item['allowed_values'] = [];
          foreach ($explode as $val) {
            $extract = explode('|', $val);
            $value = $extract[0];
            $label = !empty($extract[1]) ? $extract[1] : $extract[0];
            $item['allowed_values'][$value] = $label;
          }
        }
      }
      foreach ($item['allowed_values'] as $value => $label) {
        $structured_values[] = [
          'value' => $value,
          'label' => $label,
        ];
      }
      $settings['field_settings'][$subfield]['allowed_values'] = $structured_values;
    }
    return $settings;
  }

  /**
   * {@inheritdoc}
   */
  public static function fieldSettingsFromConfigData(array $settings): array {
    if (empty($settings['field_settings'])) {
      return $settings;
    }
    foreach ($settings['field_settings'] as $field => $subfield) {
      if (empty($subfield['allowed_values'])) {
        continue;
      }
      $structured_values = [];
      foreach ($subfield['allowed_values'] as $item) {
        $structured_values[$item['value']] = $item['label'];
      }
      $settings["field_settings"][$field]["allowed_values"] = $structured_values;
    }
    return $settings;
  }

  /**
   * Checks if list option is allowed for a given sub-field type.
   */
  public static function isListAllowed(string $subfield_type): bool {
    $list_types = array_keys(self::subfieldTypes());
    $notAllowed = [
      'boolean', 'text', 'datetime_iso8601', 'date', 'json', 'blob', 'file', 'entity_reference',
    ];
    return in_array($subfield_type, $list_types) && !in_array($subfield_type, $notAllowed);
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition): array {
    $settings = $field_definition->getSettings();

    $data = [];
    $pluginService = \Drupal::service('plugin.manager.data_field.type');
    foreach ($settings['columns'] as $subfield => $item) {

      // If allowed values are limited pick one of them from field settings.
      if ($settings[$subfield]['list']) {
        $data[$subfield] = array_rand($settings['field_settings'][$subfield]['allowed_values']);
        continue;
      }
      $type = $item['type'] ?? 'string';
      $plugin = $pluginService->createInstance($type);
      $data[$subfield] = $plugin->generateSampleValue($item);
    }

    return $data;
  }

  /**
   * Check for duplicate names on our columns settings.
   */
  public function machineNameExists($value, array $form, FormStateInterface $form_state) {
    $count = 0;
    foreach ($form_state->getValue(['settings', 'storage']) as $item) {
      if ($item['name'] == $value) {
        $count++;
      }
    }
    return $count > 1;
  }

  /**
   * Callback for both ajax-enabled buttons.
   *
   * Selects and returns the fieldset with the names in it.
   */
  public function actionCallback(array &$form, FormStateInterface $form_state) {
    return $form['settings']['storage'];
  }

  /**
   * Submit handler for the "add-one-more" button.
   *
   * Increments the max counter and causes a rebuild.
   */
  public static function addSubmit(array &$form, FormStateInterface $form_state) {
    $form_state->set('add', TRUE);
    $form_state->setRebuild();
  }

  /**
   * Submit handler for the "remove one" button.
   *
   * Decrements the max counter and causes a form rebuild.
   */
  public static function removeSubmit(array &$form, FormStateInterface $form_state) {
    $form_state->set('remove', $form_state->getTriggeringElement()['#delta']);
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public static function mainPropertyName() {
    return NULL;
  }

}
